@@ -4,68 +4,32 @@ use strict;
 use warnings;
 use Carp qw/croak/;
 use parent qw/Class::Data::Inheritable/;
-use Class::Accessor::Lite;
 use Data::Validator;
+use DBIx::Sunny::Util qw/bind_and_execute expand_placeholder/;
 use DBIx::TransactionManager;
 use DBI qw/:sql_types/;
 use Scalar::Util qw/blessed/;
 use SQL::Maker::SQLType qw/sql_type/;
-use SQL::NamedPlaceholder;
 
 $Carp::Internal{"DBIx::Sunny::Schema"} = 1;
 
-Class::Accessor::Lite->mk_ro_accessors(qw/dbh readonly/);
+use Class::Accessor::Lite 0.05 (
+    new => 1,
+    ro  => [qw/dbh readonly/],
+);
 
 __PACKAGE__->mk_classdata( '__validators' );
 __PACKAGE__->mk_classdata( '__deflaters' );
 
-sub new {
-    my $class = shift;
-    my %args = @_;
-    bless {
-        readonly => delete $args{readonly},
-        dbh => delete $args{dbh},
-    }, $class;
-};
-
 sub fill_arrayref {
     my $self = shift;
-    my ($query, @bind) = @_;
-    return if ! defined $query;
-    if (@bind == 1 && ref $bind[0] eq 'HASH') {
-        ($query, my $bind_param) = SQL::NamedPlaceholder::bind_named($query, $bind[0]);
-        return $query, @$bind_param;
-    }
-    my @bind_param;
-    my $modified_query = $query;
-    my $i=1;
-    for my $bind ( @bind ) {
-        if ( ref($bind) && ref($bind) eq 'ARRAY' ) {
-            my $array_query = substr('?,' x scalar(@{$bind}), 0, -1);
-            my $search_i=0;
-            my $replace_query = sub {
-                $search_i++;
-                if ( $search_i == $i ) {
-                    return $array_query;
-                }
-                return '?';
-            };
-            $modified_query =~ s/\?/$replace_query->()/ge;
-            push @bind_param, @{$bind};
-        }
-        else {
-            push @bind_param, $bind;
-        }
-        $i++;
-    }
-    return ($modified_query, @bind_param);
+    return expand_placeholder(@_);
 }
 
-
 sub select_one {
     my $class = shift;
     if ( ref $class ) {
-        my ($query, @bind) = $class->fill_arrayref(@_);
+        my ($query, @bind) = expand_placeholder(@_);
         my $row = $class->dbh->selectrow_arrayref($query, {}, @bind);
         return unless $row;
         return $row->[0];
@@ -88,7 +52,7 @@ sub select_one {
 sub select_row {
     my $class = shift;
     if ( ref $class ) {
-        my ($query, @bind) = $class->fill_arrayref(@_);
+        my ($query, @bind) = expand_placeholder(@_);
         my $row = $class->dbh->selectrow_hashref($query, {}, @bind);
         return unless $row;
         return $row;
@@ -116,7 +80,7 @@ sub select_row {
 sub select_all {
     my $class = shift;
     if ( ref $class ) {
-        my ($query, @bind) = $class->fill_arrayref(@_);
+        my ($query, @bind) = expand_placeholder(@_);
         my $rows = $class->dbh->selectall_arrayref($query, { Slice => {} }, @bind);
         return $rows;
     }
@@ -145,7 +109,7 @@ sub select_all {
 sub query {
     my $class = shift;
     if ( ref $class ) {
-        my ($query, @bind) = $class->fill_arrayref(@_);
+        my ($query, @bind) = expand_placeholder(@_);
         croak "couldnot use query for readonly database handler" if $class->readonly;
         my $sth = $class->prepare($query);
         return $sth->execute(@bind);
@@ -191,7 +155,7 @@ sub args {
                 $deflater{$name} = $deflater;
             }
             push @rules, $name, $rule;
-        } 
+        }
         $deflaters->{$method} = \%deflater;
         $validators->{$method} = Data::Validator->new(@rules);
     }
@@ -204,7 +168,7 @@ sub args {
         @caller_args = @DB::args;
     }
     local $Carp::CarpLevel = 3;
-    local $Carp::Internal{'Data::Validator'} = 1;   
+    local $Carp::Internal{'Data::Validator'} = 1;
     my $result = $validators->{$method}->validate(@caller_args);
 
     if ( my @deflaters = keys %{$deflaters->{$method}} ) {
@@ -253,22 +217,6 @@ sub __setup_accessor {
         }
     }
 
-    my $bind_and_execute = sub {
-        my ($dbh, $query, @binds) = @_;
-        my $sth = $dbh->prepare_cached($query);
-        my $i = 0;
-        for my $bind (@binds) {
-            if ( blessed($bind) && $bind->can('value_ref') && $bind->can('type') ) {
-                # If $bind is an SQL::Maker::SQLType or compatible object, use its type info.
-                $sth->bind_param(++$i, ${ $bind->value_ref }, $bind->type);
-            } else {
-                $sth->bind_param(++$i, $bind);
-            }
-        }
-        my $ret = $sth->execute;
-        return ( $sth, $ret );
-    };
-
     my $validator = Data::Validator->new(@rules);
 
     my $do_query = sub {
@@ -310,25 +258,18 @@ sub __setup_accessor {
         };
 
         my $query = $query_tmpl;
+        my @bind_param;
         if ($is_named_placeholder_query) {
             ($query, my $bind_param) = SQL::NamedPlaceholder::bind_named($query, $args);
-            return $bind_and_execute->($self->dbh, $query, @$bind_param)
+            @bind_param = @$bind_param;
         }
-
-        my @keys = @bind_keys;
-        my @bind_param;
-        $query =~ s{\?}{
-            my $bind = $args->{ shift @keys };
-            if (ref($bind) && ref($bind) eq 'ARRAY') {
-                push @bind_param, @$bind;
-                join ',', ('?') x @$bind;
-            } else {
-                push @bind_param, $bind;
-                '?';
-            }
-        }ge;
-
-        return $bind_and_execute->($self->dbh, $query, @bind_param)
+        else {
+            my @bind = map { $args->{$_} } @bind_keys;
+            ($query, @bind_param) = expand_placeholder($query, @bind);
+        }
+        my $sth = $self->dbh->prepare_cached($query);
+        my $ret = bind_and_execute($sth, @bind_param);
+        return ( $sth, $ret );
     };
 
     {
@@ -491,21 +432,21 @@ DBIx::Sunny::Schema - SQL Class Builder
 
 =over 4
 
-=item __PACKAGE__->select_one( $method_name, @validators, $sql );
+=item C<< __PACKAGE__->select_one( $method_name, @validators, $sql ); >>
 
-build a select_one method named $method_name with validator. validators arguments are passed for Data::Validator. you can use Mouse's type constraint. Type constraint are also used for SQL's bind type determination. 
+build a select_one method named $method_name with validator. validators arguments are passed for Data::Validator. you can use Mouse's type constraint. Type constraint are also used for SQL's bind type determination.
 
-=item __PACKAGE__->select_row( $method_name, @validators, $sql, [\&filter] );
+=item C<< __PACKAGE__->select_row( $method_name, @validators, $sql, [\&filter] ); >>
 
-build a select_row method named $method_name with validator. If a last argument is CodeRef, this coderef will be applied for a result row.
+build a select_row method named $method_name with validator. If a last argument is CodeRef, this CodeRef will be applied for a result row.
 
-=item __PACKAGE__->select_all( $method_name, @validators, $sql, [\&filter] );
+=item C<< __PACKAGE__->select_all( $method_name, @validators, $sql, [\&filter] ); >>
 
-build a select_all method named $method_name with validator. If a last argument is CodeRef, this coderef will be applied for all result row.
+build a select_all method named $method_name with validator. If a last argument is CodeRef, this CodeRef will be applied for all result row.
 
-=item __PACKAGE__->query( $method_name, @validators, $sql );
+=item C<< __PACKAGE__->query( $method_name, @validators, $sql ); >>
 
-build a query method named $method_name with validator.  
+build a query method named $method_name with validator.
 
 =back
 
@@ -540,7 +481,7 @@ If you want to deflate argument before execute SQL, you can it with adding defla
       body => 'Str',
       created_on => { isa => 'DateTime', deflater => sub { shift->strftime('%Y-%m-%d %H:%M:%S')  },
       <<SQL);
-  INSERT INTO articles (subject, body, created_on) 
+  INSERT INTO articles (subject, body, created_on)
   VALUES ( ?, ?, ?)',
   SQL
 
@@ -550,51 +491,51 @@ If you want to deflate argument before execute SQL, you can it with adding defla
 
 =over 4
 
-=item new({ dbh => DBI, readonly => ENUM(0,1) ) :DBIx::Sunny::Schema
+=item C<< new({ dbh => DBI, readonly => ENUM(0,1) ) >> :DBIx::Sunny::Schema
 
-create instance of schema. if readonly is true, query method's will raise exception.
+create instance of schema. if C<readonly> is true, query method's will raise exception.
 
-=item dbh :DBI
+=item C<dbh> :DBI
 
-readonly accessor for DBI database handler. 
+C<readonly> accessor for DBI database handler.
 
-=item select_one($query, @bind) :Str
+=item C<select_one($query, @bind)> :Str
 
 Shortcut for prepare, execute and fetchrow_arrayref->[0]
 
-=item select_row($query, @bind) :HashRef
+=item C<select_row($query, @bind)> :HashRef
 
 Shortcut for prepare, execute and fetchrow_hashref
 
-=item select_all($query, @bind) :ArrayRef[HashRef]
+=item C<select_all($query, @bind)> :ArrayRef[HashRef]
 
 Shortcut for prepare, execute and selectall_arrayref(.., { Slice => {} }, ..)
 
-=item query($query, @bind) :Str
+=item C<query($query, @bind)> :Str
 
 Shortcut for prepare, execute. 
 
-=item txn_scope() :DBIx::TransactionManager::Guard
+=item C<txn_scope()> :DBIx::TransactionManager::Guard
 
 return DBIx::TransactionManager::Guard object
 
-=item do(@args) :Str
+=item C<do(@args)> :Str
 
-Shortcut for $self->dbh->do()
+Shortcut for C<< $self->dbh->do() >>
 
-=item prepare(@args) :DBI::st
+=item C<prepare(@args)> :DBI::st
 
-Shortcut for $self->dbh->prepare()
+Shortcut for C<< $self->dbh->prepare() >>
 
-=item func(@args) :Str
+=item C<func(@args)> :Str
 
-Shortcut for $self->dbh->func()
+Shortcut for C<< $self->dbh->func() >>
 
-=item last_insert_id(@args) :Str
+=item C<last_insert_id(@args)> :Str
 
-Shortcut for $self->dbh->last_insert_id()
+Shortcut for C<< $self->dbh->last_insert_id() >>
 
-=item args(@rule) :HashRef
+=item C<args(@rule)> :HashRef
 
 Shortcut for using Data::Validator. Optional deflater arguments can be used.
 Data::Validator instance will cache at first time.
@@ -611,7 +552,7 @@ Data::Validator instance will cache at first time.
       $arg->{id} ...
   }
 
-$args is validated arguments. @_ is not needed.
+C<$args> is validated arguments. C<@_> is not needed.
 
 =back
 
